  /* =============================================================================
   * UN-comment "#define FORMAT_SPIFFS" to: format SPIFFS-mem. and create a new DB
   * COMMENT this if using a early created DB
   * ============================================================================= */

/** define the user db record and fields parameters **/
const char   db_Name[31] = "/simple.db",   // if using SPIFFS then the '/' is necessary
           db_Backup[31] = "/simple.cdb";  // backup to SD or SPIFFS

/** add/remove fields and set width and the field names **/
/** simpleDB record format:
 *  key.|T|field1....|field2.........|field3...|....ad/remove fields.....|
 *  | = fields delimiter
 *  T = tombstone flag
 */
const uint8_t db_KeyLen = 4,         // set the FIXED key len
              field1Len = 10,         // set max field len ...
              field2Len = 20,         // ... add/remove fields
              field3Len = 30,
              field4Len = 40,
              db_RecLen =  db_KeyLen + 1 +  // len plus one fields delimiter position
                                 2 +  // FIXED tombstone flag plus one delimiter position
                     field1Len + 1 +  // add/remove field len ...
                     field2Len + 1 +  //   ... must correspond with the above declared nummbers field<?>Len fields
                     field3Len + 1 +
                     field4Len + 1;
char recordStr[db_RecLen + 1];           // to set the read/write buffer
const uint16_t db_MaxRec = 512;          // max number of records
// first declaration, then #include "sDB_SD.h"
#include <sDB_SPIFFS.h>

typedef struct RecFields{  /** define the table value holders **/
    char key[db_KeyLen + 1];             // do NOT change the name "key"
    char field1[field1Len + 1];       // change te name and add/remove fields ...
    char field2[field2Len + 1];       //   ... must correspond with the above declared nummbers field<?>Len fields
    char field3[field3Len + 1];   
    char field4[field4Len + 1];
};  

/** the list of simpleDB funtions
 *  =============================
    ** specific simpleDB funtions **
    uint8_t dtW;   // max data width in a record, exclusive 'key' and 'tombstone' marker
    uint16_t dbR;  // record counter, exlusive tombstone records
    char *dbB;     // record read buffer

    ** init an new or existing database **
    DB_RESULT initDB(const char *db_Name, const char *db_Backup, uint8_t db_KeyLen, uint8_t db_RecLen, uint16_t db_MaxRec);
    ** simpleDB functions **
    DB_RESULT nrRec();                                      // set counter dbR, the nummer of records, tombstone records are excluded
    DB_RESULT addRec(const char *key, const char *data);    // add a record with a UNIQUE key 
    DB_RESULT writeRec(const char *key, const char *data);  // save a record, overwrite if key exist else add a record 
    DB_RESULT readRec(const char *key);                     // read the record with the given "key", it is in record buffer dbB 
    DB_RESULT getRec(uint8_t nr);                           // read a record on position "nr", it is in record buffer dbB 
    DB_RESULT findRec(const char *key);                     // find record with the given "key", slow but simple and fast enough for a small db 
    DB_RESULT delRec(const char *key);                      // tombstone a record, remove tombstones with comprDb 
    DB_RESULT cmprDb();                                     // copy db, then remove tombstone records 
    DB_RESULT openDB();                                     // test if open, else open or create DB
              closeDB();                                    // close db
    
    DB_RESULT dbResult;                                     // DB_RESULT error code holder
    
    enum DB_RESULT {  // result code
      DB_OK       =  0,  // OK, no error
      DB_EMPTY    =  1,  // no record(s) in db
      NOT_FOUND   =  2,  // record not found
      REC_EXIST   =  3,  // record already exists
      // ERRORs //
      DIF_NR      =  4,  // difference in number of records (DB and backup DB)
      CORRUPT     =  5,  // db corrupt
      MAX_CHARS   =  6,  // key or data: too many characters
      MAX_REC     =  7,  // max records in db
      SEEK_ERR    =  8,  // file seek error  
      EXIST_ERR   =  9,  // no file
      BACKUP_ERR  = 10,  // backup problem
      FORMAT_ERR  = 11,  // format problem
      TOMBSTONE   = 12,  // deleted record
      REMOVE_ERR  = 13,  // delete file error
      RENAME_ERR  = 14,  // rename file error
      MOUNT_ERR   = 15,  // can not mount  
      OPEN_ERR    = 16,  // file open error  
      DB_ERR      = 17,  // db error
};
**/  

RecFields recIn;   // IN buffer structure (write)
RecFields recOut;  // OUT buffer structure (read)
DB_RESULT result;  // DB functions result holder (OK, ERROR, ...)

void setRec(char *_key, char *_field1, char *_field2, char *_field3, char *_field4);
DB_RESULT dbWriteRec();
DB_RESULT dbReadRec();
void listDir(fs::FS &fs, const char * dirname, uint8_t levels);

char codeList[18][11] = {
  "DB_OK",
  "DB_EMPTY",
  "NOT_FOUND",
  "REC_EXIST",
  "DIF_NR",
  "CORRUPT",
  "MAX_CHARS",
  "MAX_REC",
  "SEEK_ERR",
  "EXIST_ERR",
  "BACKUP_ERR",
  "FORMAT_ERR",
  "TOMBSTONE",
  "REMOVE_ERR",
  "RENAME_ERR",
  "MOUNT_ERR",
  "OPEN_ERR",
  "DB_ERR"
};

void setup() {
  Serial.begin(115200);
//  Serial.setDebugOutput(true); 
  Serial.println("\n...\n");

  /** for this demo delete a existed db **/
//  result = initDB(db_Name, db_Backup, db_KeyLen, db_RecLen, db_MaxRec);
//  closeDB();
//  SPIFFS.remove((char*)db_Name);
  
  /** init and open dtabase **/
  if (initDB() == MOUNT_ERR) {
    result = DB_OK;
    Serial.printf("format SPIFFS ...");
    if (!SPIFFS.format()) result = FORMAT_ERR;
    Serial.printf("FORMAT result, SPIFFS.format():\t%s\n", codeList[result]);
    if (result == DB_OK) result = initDB();
    Serial.printf("INIT DB result, initDB:\t%s\n", codeList[result]);
    if (result != DB_OK) while (1) {;}
  }
 
  /** make end save a record using RecFields recIn.<fields> **/
 Serial.printf(">>> overwrite or add a record using structure 'recIn' and function 'dbWriteRec'...\n\n");
  setRec("keyA", "field1A", "field2A", "field3A", "field4A"); // set record fields ...
 Serial.printf("\trecIn.key:\t%s\n\trecIn.field1:\t%s\n\trecIn.field2:\t%s\n\trecIn.field3:\t%s\n\trecIn.field4:\t%s\n", 
           recIn.key, recIn.field1, recIn.field2, recIn.field3, recIn.field4);
  result = dbWriteRec();  // ... and save it, add or, if key exist, overwrite record
 Serial.printf("result, dbWriteRec():\t%s\n\n", codeList[result]);

  /** overwrite or add a record with a string **/
 Serial.printf(">>> overwrite or add a record using a 'string' and function 'writeRec'...\n\n");
  char str[dtW + 1];
  sprintf(str, "%s|%s|%s|%s|", "field1B", "field2B", "field3B", "field4B");
 Serial.printf("\"%s\"\n", str);
  result = writeRec("keyB", str);  // add or, if key exist, overwrite record
 Serial.printf("result, writeRec(\"keyB\"): %s\n\n", codeList[result]);

  /**  add a record  **/
 Serial.printf(">>> add a record with function 'addRec'...\n\n");
  sprintf(str, "%s|%s|%s|%s|", "field1C", "field2C", "field3C", "field4C");
 Serial.printf("\"%s\"\n", str);
  result = addRec("keyC", str);  // add or, if key exist then error
 Serial.printf("result, addRec(\"keyC\"):\t%s\n\n", codeList[result]);

 Serial.printf(">>> # of records in DB 'uint16_t  dbR'...\n\n");
 Serial.printf("records in DB:\t%d\n\n", dbR); 

  /** find a record **/
 Serial.printf(">>> find a record with function 'findRec' (test if record exists)...\n\n");
  result = findRec("keyB");
 Serial.printf("result, findRec(\"keyB\"): %s\n\n", codeList[result]);

  /** get a record (incl tomstone) **/
 Serial.printf(">>> get a record (incl tomstone) with function 'getRec'...\n\n");
  result = getRec(1);
 Serial.printf("result, getRec(1):\t%s, %s\n", codeList[result], dbB);
  result = getRec(2);
 Serial.printf("result, getRec(2):\t%s, %s\n", codeList[result], dbB);
  result = getRec(0);
 Serial.printf("result, getRec(0):\t%s, %s\n", codeList[result], dbB);
 Serial.printf(">>> there is not a record 4...\n\n");
  result = getRec(4);
 Serial.printf("result, getRec(4):\t%s, %s\n\n", codeList[result], dbB);

  /** delete a record (set tomstone) **/
 Serial.printf(">>> delete a record (set tomstone) with function 'delRec'...\n\n");
  result = delRec("keyB");
 Serial.printf("result, delRec(\"keyB\"):\t%s\n", codeList[result]);
 Serial.printf(">>> tombstone is set to \"1\"...\n\n");
  result = getRec(1);
 Serial.printf("result, getRec 1:\t%s, %s\n", codeList[result], dbB);
 Serial.printf(">>> no tombstone in list...\n\n");

 Serial.printf(">>> using structure 'recOut' and function 'dbReadRec'...\n\n");
  /** first set the "key" in recOut.key **/
  strcpy(recOut.key, "keyC");
  result = dbReadRec();
 Serial.printf("\trecOut.key:\t%s\n\trecOut.field1:\t%s\n\trecOut.field2:\t%s\n\trecOut.field3:\t%s\n\trecOut.field4:\t%s\n", 
          recOut.key, recOut.field1, recOut.field2, recOut.field3, recOut.field4);
 Serial.printf("result, dbReadRec():\t%s\n\n", codeList[result]);

 Serial.printf(">>> copy db and remove tombstone records with function 'cmprDb'...\n\n");
  result = cmprDb();
 Serial.printf("result, cmprDb():\t%s\n\n", codeList[result]);

 Serial.printf(">>> # of records in DB 'uint16_t  dbR'...\n\n");
 Serial.printf("records in DB:\t%d\n\n", dbR); 
  
  closeDB();

  /** list files **/
  Serial.printf(">>> list of all the files...\n\n");
  listDir(SPIFFS, "/", 0);
  SPIFFS.end();  // Use this method before updating SPIFFS using OTA
}

void loop() {
}

void setRec(char *_key, char *_field1, char *_field2, char *_field3, char *_field4) {
  strcpy(recIn.key, _key);
  strcpy(recIn.field1, _field1);  // change and add/remove RecFields names
  strcpy(recIn.field2, _field2);
  strcpy(recIn.field3, _field3);
  strcpy(recIn.field4, _field4);
}

/** write a record from RecFields names **/
DB_RESULT dbWriteRec() {
  char str[dtW + 1];
  /** get a simpleDB 'record' string and save it **/
  // change and add/remove RecFields names
  sprintf(str, "%s|%s|%s|%s|", recIn.field1, recIn.field2, recIn.field3, recIn.field4);
  return writeRec(recIn.key, str);  // add or, if key exist, overwrite record 
}

/** read a record and insert it in RecFields recOut.names **/
DB_RESULT dbReadRec() { 
  char *v, *b;
  uint8_t w = dtW;
  if (readRec(recOut.key) == DB_OK) { // record is in dbB
    while (dbB[--w] == ' ') dbB[w] = '\0';  // remove trailing white-space (added by addRec())
    v = strtok_r(dbB, "|", &b);
    strcpy(recOut.field1, v);   // change and add/remove RecFields names
    v = strtok_r(NULL, "|", &b);
    strcpy(recOut.field2, v);
    v = strtok_r(NULL, "|", &b);
    strcpy(recOut.field3, v);
    v = strtok_r(NULL, "|", &b);
    strcpy(recOut.field4, v);
  }
  return dbResult;
}

void listDir(fs::FS &fs, const char * dirname, uint8_t levels){
    Serial.printf("Listing directory: %s\r\n", dirname);

    File root = fs.open(dirname);
    if(!root){
        Serial.println("- failed to open directory");
        return;
    }
    if(!root.isDirectory()){
        Serial.println(" - not a directory");
        return;
    }

    File file = root.openNextFile();
    while(file){
        if(file.isDirectory()){
            Serial.print("  DIR : ");
            Serial.println(file.name());
            if(levels){
                listDir(fs, file.name(), levels -1);
            }
        } else {
            Serial.print("  FILE: ");
            Serial.print(file.name());
            Serial.print("\tSIZE: ");
            Serial.println(file.size());
        }
        file = root.openNextFile();
    }
}
