#include <sDateTimeSun.h>    // www.kendziorra.nl (simpleLib)

const float lat = 52.0881752, // Nederlands Spoorwegmuseum
            lon = 5.1304144,  // Utrecht, EU-NL
            tz  = 3600.0,     // offset local - UTC/GMT in sec.
            tmp = 18.0,       // temperature *C
            prs = 1002.0;     // pressure
char list[33][19] = {  // test values
// Nederlands Spoorwegmuseum
// NL, Utrecht
// 52.0881752, 5.1304144
// 6 dec 2018, 8:30-16:30
// list from: https://www.sunposition.info/sunposition/spc/locations.php
  "08:30  126*  -1*",// time azim.  alti.   time azim.  alti.   time azim. alti.
  "08:45  129*   1*", "09:00  132*   2*", "09:15  135*   4*", "09:30  138*   6*", 
  "09:45  142*   7*", "10:00  145*   9*", "10:15  148*  10*", "10:30  151*  11*", 
  "10:45  155*  12*", "11:00  158*  13*", "11:15  162*  14*", "11:30  165*  14*", 
  "11:45  169*  15*", "12:00  172*  15*", "12:15  176*  15*", "12:30  179*  15*", 
  "12:45  183*  15*", "13:00  187*  15*", "13:15  190*  15*", "13:30  194*  14*", 
  "13:45  197*  14*", "14:00  201*  13*", "14:15  204*  12*", "14:30  207*  11*", 
  "14:45  211*  10*", "15:00  214*   9*", "15:15  217*   7*", "15:30  220*   6*", 
  "15:45  224*   4*", "16:00  227*   3*", "16:15  230*   1*", "16:30  233*  -1*"};             

void setup() {
  Serial.begin(115200);
  while (!Serial); delay(1000);
  Serial.println("...\n");
  struct sEpoch ep = sMakeTime(8, 30, 0, 0, 6, 12, 2018);  // make a timestamp - simpleLib "sDateTimeSun.h"
  time_t t = ep.sec;  // simpleLib "sDateTimeSun.h"
  sSolPosIn sunP;
  sunP.lat = lat;
  sunP.lon = lon;
  sunP.temp = tmp;  // we use a fixed value for this demo
  sunP.pres = prs;  // we use a fixed value for this demo
  sunP.zone = tz;   // offset from UTC/GMT, use my simpleLib "sDateTimeSun.h" sTimeDate function to get dt.offset
  Serial.println("Nederlands Spoorwegmuseum, NL, Utrecht\n52.0881752, 5.1304144\n 6 dec 2018, 8:30-16:30\nChecklist from: http://www.sunposition.info/sunposition/spc/locations.php");
  Serial.printf ("%d-%02d-%02d lat: %f, lon: %f\n      azim    alti   (   checklist    )\n", year(t), month(t), day(t), lat, lon);
  uint8_t count = 0;
  while((hour(t) * 60 + minute(t)) <= (16 * 60 + 30)) {  // 16:30? stop!
    sunP.time = t;
    //sunP.temp = tmp;  // use my simpleLib "sOpenweathermap.h" API (get a free key)
    //sunP.pres = prs;  // use my simpleLib "sOpenweathermap.h"
    sSolPosOut su = sSolPos(sunP);  // azimuth: degrees relative to north, alltitude: degrees relative to horizontal line 
    Serial.printf ("%02u:%02u %5.2f* %5.2f* (%s)\n", hour(t), minute(t), su.azimuth, su.altitude, list[count++]);
    t += 15 * sOneminute; // add 15 minutes
  }
  Serial.println("Between sunset and sunrise the azimuth/altitude show -2*.");
}

void loop() {
}
