/**   Download a LOG-FILE from    *
  * SD-Card, SPIFFS-MEM, FFat-MEM *
  *  HTML example code in folder  *
  *-------------------------------*
  * download my simpleLib library *
  * www.kendziorra.nl             *
  *-------------------------------**/
  
/////////////////////////////////////
// HTML download in project folder //
//   - direct link.html            //
//   - downloadLogFile.html        // 
/////////////////////////////////////

//** START sWiFiFull.h definitions **/
#define USE_STA        // use WiFi mode STA
#ifdef USE_STA         // only station mode
  #define USE_STAIP    // if using a static STA IP  (is needed with: USE_WS)
//  #define USE_STADNS   // use WiFi STA DNS (use it if USE_STAIP and USE_TIME)
//  #define USE_STACHAN  // used in AP_STA mode
//  #define USE_OTA      // ESP32 OTA
//  #define USE_TIME     // NTP, date, time, timezone, DST/Summertime, Sunrise, Sunset /** my simpleLIB library: sDateTimeSun.h **/
//  #define USE_WS       // WebSockets Server /** library: https://github.com/Links2004/arduinoWebSockets **/
//  #define USE_WTHR     // Weather  /** my simpleLIB library: sOpenweathermap.h **/
#endif
//#define USE_AP         // use WiFi mode AP
#ifdef USE_AP          // only softAp mode
  #define USE_APIP     // if using a static AP IP 
  #define USE_APCHAN   // used in AP_STA mode
  #define USE_APHID    // SSID hidden
  #define USE_APMAXCON // max connections, default 4, max 10
  #define USE_NOW      // ESP-NOW only in softAP mode
#endif
#if defined(USE_NOW) || defined(USE_WS)
//  #define USE_PARSER   // parse the received message to fieldnames an fieldValues 
#endif
#if defined(USE_STA) && defined(USE_AP) 
  #define USE_LR       // ESP32 softAP Long Range mode up to 1000m (only in AP_STA mode)
#endif

///////////////////////////////////////////////////////////////
//#define USE_SD       // UNcomment this for using "SD-card"
//#define USE_SPIFFS   // UNcomment this for using "SPIFFS-mem"
//#define USE_FFAT     // UNcomment this for using "FFAT-mem"
///////////////////////////////////////////////////////////////
#ifdef USE_SD
  #include <SD.h> 
  #include <SPI.h>
  uint8_t ssPin = 5;
  String txt = "SD"; 
#elif defined(USE_SPIFFS)
  #include <SPIFFS.h>
  String txt = "SPIFFS"; 
#elif defined(USE_FFAT)
  #include <FFat.h>
  String txt = "FFat"; 
#endif

///////////////////////////////////////////////////////////////////////////////////////////////////
// ESP32 SD.open(..., ...): FILE_WRITE overwrites existing data, for adding use FILE_APPEND !!!! //
//                          ==================================================================== //
///////////////////////////////////////////////////////////////////////////////////////////////////

const char *staSsid = "edit-edit"@,
       *staPassword = "edit-edit"@;
const IPAddress staIP(192, 168, 178, 121)@,  // edit IP also in HTML
         staGatewayIP(192, 168, 178,   1),
          staSubnetIP(255, 255, 255,   0);
const uint8_t wsPort = 80;
// after the declarations!!!
#include <sWiFiFull.h>

bool logPresent = false;
String logFile = "log.txt",  // log file name - name and path can be changed in the code
       logPath = "/";        // path to log file (at least one '/')
// after the log-file declarations!!!
#include "sDownloadLogFile.h"

void setup(void){
  Serial.begin(115200);
  while (!Serial); delay(1000);
  Serial.println("\n");
  setupWiFi();  // is necessary on this place - init all your defined choices
  #ifdef USE_SD
    logPresent = SD.begin(ssPin);  // format on PC or ....
    /////// DEMO CODE firts time write a file ///////
//    logFile = "logP1.txt";  // change filename (and/or path)
    if (logPresent && !SD.exists(logPath + logFile)) {  // write new file
      File file = SD.open(logPath + logFile, FILE_WRITE);  // FILE_APPEND);
      file.print("TEST: " + txt);
      file.close();
    } ///////
  #elif defined(USE_SPIFFS)
    logPresent = SPIFFS.begin();
    /////// DEMO CODE firts time write a file ///////
//    logFile = "logP1.txt";  // change filename (and/or path)
    if (!logPresent) {
      if (!SPIFFS.begin(true)) {  // format and write new file
        logPresent = true;
        File file = SPIFFS.open(logPath + logFile, FILE_WRITE);  // FILE_APPEND);
        file.print("TEST: " + txt);
        file.close();
      }
    } ///////
  #elif defined(USE_FFAT)
    logPresent = FFat.begin();
    /////// DEMO CODE firts time write a file ///////
//    logFile = "logP1.txt";  // change filename (and/or path)
    if (!logPresent) {
      if (!FFat.begin(true)) {  // format and write new file
        logPresent = true;
        File file = FFat.open(logPath + logFile, FILE_WRITE);  // FILE_APPEND);
        file.print("TEST: " + txt);
        file.close();
      }
    } ///////
  #endif
  if (logPresent) {
    server.on("/download", downloadLog);
    server.begin();
    Serial.print("use: http://");
    Serial.print(WiFi.localIP());
    Serial.println("/download");
  }
  else Serial.println("No file system!");
}

void loop(void){
  loopWiFi();  // is necessary on this place
  if (logPresent) server.handleClient(); // Listen for client connections
}