#ifndef _MY_MIDEA_AC_H_
  #define _MY_MIDEA_AC_H_

  #include "editMideaAC.h"
  // next libraries after "editMideaAC.h" !!
  #include <EEPROM.h>
  #include <sDS18B20.h>  // (simpleLib)

  byte inTemp = 20,  // start values
      outTemp = 20,  //  "    "
  savedMinute = 99,  // saved action minutes
  repeatCount = 0,
         yoyo = 0;
  boolean aircoInit = false,
          aircoIsOn = false,
            actionT = true;  // action on (re)start
  char lastAction[33] = "|Nothing!|";  // from actionTime()
  time_t t;  // time from now

  sDS18B20 sTmp(tmpPin, tmpComp, tmpMode);
  MIDEA ac;  // struct: values in use

  void putMidea(boolean def = false);  // true: use default values
  void getMidea();  // get actual values
  void doMidea(char *msgBuf, byte len);  // command from user
  void wsAction();

  void putMidea(boolean def) {  // save actual values or, it true, default values
    EEPROM.put(0, "MiDeA");  // do not chance!
    if (def) {  // save default values
      MIDEA val;
      EEPROM.put(6, val);
    }
    else EEPROM.put(6, ac);  // save actual values
    EEPROM.commit();
  }

  void getMidea() {  // load actual values
    char m[6];
    EEPROM.get(0, m);
    if (strcmp(m, "MiDeA") == 0) EEPROM.get(6, ac);
    else putMidea(true);  // no values in EEPROM, save and load default values
  }

  void doMidea(char *msgBuf, byte len) {
    if (len > 40) {
      webSocket.broadcastTXT("Wrong length!");
      return;
    }
    if (strcmp(msgBuf, "|ACT|") == 0) {
      webSocket.broadcastTXT(lastAction);
      return;
    }
    if (strcmp(msgBuf, "|DEF|") == 0) {
      putMidea(true);  // set basic midea values
      getMidea();      // load values in airco
      webSocket.broadcastTXT("|Default values loaded.|");
      return;
    }
    inTemp = (byte)sTmp.getTemp();  // temperature room - read temp for user APP
    if (getWeather()) outTemp = atoi(owm.temp);  // temperature outside
    else outTemp = 99;  // no Weather
    char strVal[45];
    sprintf(strVal, "|%u|%u|%u|%u|%u|%u|%u|%u|%u|%u|%u|%u|%u|",
            ac.stat, (aircoIsOn == 99 ? 0 : aircoIsOn), inTemp, outTemp,
            ac.hhOn, ac.mmOn,
            ac.hhOff, ac.mmOff,
            ac.tmpIn, ac.tmpOut,
            ac.mode, ac.fan, ac.cmd);
    if (strcmp(msgBuf, "|?|") == 0) webSocket.broadcastTXT(strVal);  // info request
    else if (strcmp(msgBuf, strVal) != 0) {  // changed values?
      byte fldIn[13];
      String str = String(msgBuf);
      byte i = 0,
          p1 = -1,
          p2 = -1;
      for (byte i = 0; i < 13; i++) fldIn[i] = 99;  // fill as not active
      p1 = str.indexOf('|');
      while (p1 >= 0 && i < 13) {
        p2 = str.indexOf('|', p1 + 1);
        if ((p2 - p1) > 1) {
          String s = str.substring(p1 + 1, p2);
          fldIn[i] = s.toInt();
        }
        i++;
        p1 = str.indexOf('|', p2);
      }
      if (i == 13) {  // 13 fields?
        byte i = 0;
        boolean f = false;
        if (fldIn[i]   != 99 && fldIn[i] <=  2 &&                   fldIn[i] != ac.stat)   {ac.stat   = fldIn[i]; f = true;}
        i = 4;
        if (fldIn[i]   != 99 && fldIn[i] >=  6 && fldIn[i] <= 23 && fldIn[i] != ac.hhOn)   {ac.hhOn   = fldIn[i]; f = true;}
        if (fldIn[++i] != 99 && fldIn[i] >=  0 && fldIn[i] <= 59 && fldIn[i] != ac.mmOn)   {ac.mmOn   = fldIn[i]; f = true;}
        if (fldIn[++i] != 99 && fldIn[i] >=  6 && fldIn[i] <= 23 && fldIn[i] != ac.hhOff)  {ac.hhOff  = fldIn[i]; f = true;}
        if (fldIn[++i] != 99 && fldIn[i] >=  0 && fldIn[i] <= 59 && fldIn[i] != ac.mmOff)  {ac.mmOff  = fldIn[i]; f = true;}
        if (fldIn[++i] != 99 && fldIn[i] >= 17 && fldIn[i] <= 30 && fldIn[i] != ac.tmpIn)  {ac.tmpIn  = fldIn[i]; f = true;}
        if (fldIn[++i] != 99 && fldIn[i] >= 20 && fldIn[i] <= 50 && fldIn[i] != ac.tmpOut) {ac.tmpOut = fldIn[i]; f = true;}
        if (fldIn[++i] != 99 && fldIn[i] <=  2 &&                   fldIn[i] != ac.mode)   {ac.mode   = fldIn[i]; f = true;}
        if (fldIn[++i] != 99 && fldIn[i] <=  4 &&                   fldIn[i] != ac.fan)    {ac.fan    = fldIn[i]; f = true;}
        if (fldIn[++i] != 99 && fldIn[i] <=  5 &&                   fldIn[i] != ac.cmd)    {ac.cmd    = fldIn[i]; f = true;}
        webSocket.broadcastTXT("|Done (incorrect values have been ignored)|");
        if (f){
          putMidea();  // save the actual values
          repeatCount = yoyo = 0;
          actionT = true;
        }
      }
      else webSocket.broadcastTXT("|Expect 13 values: send a |?| command to get an example.|");
    }
    else webSocket.broadcastTXT("|There was nothing to change|");
  }

  void wsAction() {
    if (wsBuf.startsWith("|") && wsBuf.endsWith("|")) {
      byte len = wsBuf.length();
      char msg[len +1];
      strcpy(msg, wsBuf.c_str());
      doMidea(msg, len);
    }
    wsVal = false;
  }

#endif
