/**  myMideaIR.ino: A very small and simple to use AC Infrared Emitter application
  *  Totally free to use and modify. Ewald  Kendziorra sr, 2020, www.kendziorra.nl
  *  -----------------------------------------------------------------------------
  *  values: from my Midea Comfee remote control
  *  decoded with my "receiverIR.ino"  (on my web site)
  *  read: DECODING INFRARED PROTOCOLS (on my web site)
  *  read: MIDEA AC IR PROTOCOL        (on my web site)
  **/// use a OPEN-SMART 38KHz Infrared Emitter //
     //   NO NEED FOR A INFRARED LIBRARY ;=)    //
     //=========================================//
/** only 2 commands to control your Midea AC
  stdCmd(fan, tmp,  mode);  // standardCommand, use the 'enum' labels
       fan:  lowFan, midFan, highFan, autoFan
       tmp:  tmp17, tmp18, tmp19, tmp20, tmp21, tmp22, tmp23, tmp24,
             tmp25, tmp26, tmp27, tmp28, tmp29, tmp30, tmpOff
      mode:  coolMod, heatMod, autoMod
  spcCmd(cmd);              // SpesialCommand, use the 'enum' labels
       cmd:  onAC, offAC, deflectAC, oscilAC, turboAC
 **/// NOT ALL LABELS ARE DISPLAED //

#include "myMideaAC.h"


@// we use my sWiFiFull.h lib.: a very easy to use ESP32 WiFi, static IP, OTA, Time, Weather and WeSocket library
 // ===================== first to do: download my simpleLib: http://www.kendziorra.nl/download-pagina

void setup() {
  Serial.begin(115200);
  while (!Serial); delay(1000);
  Serial.println("\n...\n");
  setupWiFi();  // in sWiFiFull.h: is necessary on this place - init all your defined choices
  EEPROM.begin(32);
  getMidea();  // EEPROM read last used midea values
  delay(50);
  inTemp = sTmp.getTemp();
  if (getWeather()) outTemp = atoi(owm.temp);  // temperature outside
  else outTemp = inTemp + 1;  // no Weather temp
  mideaBegin(irPin);
}

void loop() {
  loopWiFi();  // in sWiFiFull.h: is necessary on this place - connection 'stays alive' control
  if (wsVal) wsAction();  // WebSocket data received
  t = now();
  if(actionT || (savedMinute != minute(t))) actionTime();  // action time
}

void actionTime() {  //  Midea switch time?
  static byte saveTemp = 22,
          countWeather =  0;
  uint16_t timeOn = ac.hhOn  * 60 + ac.mmOn,
          timeOff = ac.hhOff * 60 + ac.mmOff,
          minutes = hour(t)  * 60 + minute(t);
  boolean f = false;           // off (ac.stat == 1)
  actionT = false;
  savedMinute = minute(t);
  inTemp = (byte)sTmp.getTemp();  // temperature room
  if (minutes == timeOn || minutes == timeOff) repeatCount == 0;
  if (repeatCount) {
    if (repeatCount++ == 1) {  // repeat after one minutete
      if (ac.cmd) {  // repeat a special command?
        spcCmd(ac.cmd);
        delay(500);
        ac.cmd = 0;
        putMidea();
      }
      if(aircoIsOn) spcCmd(onAC);
      else spcCmd(offAC);
      return;
    }
    if (aircoIsOn && (repeatCount == 5)) {  // after five minutetes on equal temperature? repeat
      if (inTemp == saveTemp) spcCmd(onAC);
      return;
    }
    if (repeatCount == 5) repeatCount = 0;
  }
  if (++countWeather == 5) {  // 5 minutes?
    if (getWeather()) outTemp = atoi(owm.temp);  // temperature outside
    else outTemp = inTemp + 1;  // no Weather temp
    countWeather = 0;
  }
  if (yoyo) {  // anti yoyo minutes delay
    if (yoyo++ == antiYoyo) yoyo = 0;
    return;
  }
  if (ac.stat == 2) f = true;  // on
  else if (ac.stat == 0) {     // auto
    if (minutes >= timeOn && minutes < timeOff) {
      if ((ac.mode - 1) == coolMod) {
        if (outTemp >= ac.tmpOut && inTemp > ac.tmpIn) f = true;  // in cool mode
      }
      else if (inTemp < ac.tmpIn) f = true;  // in heat mode
    }
  }
  if (ac.cmd) {
    spcCmd(ac.cmd);
    delay(500);
  }
  if (!aircoInit || (f != aircoIsOn)) {
    aircoIsOn = f;
    repeatCount = yoyo = 1;
    saveTemp = inTemp;
    if (aircoIsOn) stdCmd(ac.fan - 1, ac.tmpIn - 17, ac.mode - 1);
    else spcCmd(offAC);
    sprintf(lastAction, "|Last action %02u:%02u, airco: %s|", hour(t), minute(t), (aircoIsOn ? "ON" : "OFF"));
    webSocket.broadcastTXT(lastAction);
  }
  aircoInit = true;
}
