/**  sWifiFull: a very easy to use ESP32 WiFi library.
  *  - Automatic initialization and connection
  *  - 'stays alive' control
  *  - eleven freely selectable options
  *  Totally free to use and modify
  *  Ewald Kendziorra sr, 2020, www.kendziorra.nl 
  ****************************************************/ 

#define USE_STA        // use WiFi mode STA
#ifdef USE_STA         // only station mode
  #define USE_STAIP    // if using a static STA IP (is needed with: USE_WS)
  #define USE_STADNS   // use WiFi STA DNS
  #define USE_STACHAN   // used in AP_STA mode
//  #define USE_OTA      // ArduinoOTA
//  #define USE_TIME     // NTP, date, time, timezone, DST/Summertime /** my simpleLIB library: sDateTimeSun.h       **/
//  #define USE_WS       // WebSockets Server         /** library: https://github.com/Links2004/arduinoWebSockets **/
//  #define USE_WTHR     // Weather, Sunrise, Sunset                  /**                       sOpenweathermap.h **/
#endif
#define USE_AP         // use WiFi mode AP
#ifdef USE_AP          // only softAp mode
  #define USE_APIP     // if using a static AP IP 
  #define USE_APCHAN   // used in AP_STA mode
  #define USE_APHID    // SSID hidden
  #define USE_APMAXCON // max connections, default 4, max 10
//  #define USE_NOW      // ESP-NOW only in softAP mode
#endif
#if defined(USE_NOW) || defined(USE_WS)
//  #define USE_PARSER   // parse the received message to fieldnames an fieldValues 
#endif
#if defined(USE_STA) && defined(USE_AP) 
  #define USE_LR       // ESP32 softAP Long Range mode up to 1000m (only in AP_STA mode)
#endif

const char staSsid[] = "EDIT_EDIT"@,
       staPassword[] = "EDIT_EDIT"@;
const IPAddress staIP(192, 168, 178, 172),
         staGatewayIP(192, 168, 178,   1),
          staSubnetIP(255, 255, 255,   0);
const IPAddress staPrimDNS(8, 8, 8, 8),
                   staSecDNS(8, 8, 4, 4);
const uint8_t staChannel = 10;  // set this channel FIXED in your ROUTER
const char apSsid[] = "apSsid1",
       apPassword[] = "apPassword1";
const uint8_t apChannel = staChannel;
const uint8_t apMaxCon = 6;  //  default 4, max 10
const bool apHiden = true;
const IPAddress apIP(192, 168,   4, 5),
         apGatewayIP(192, 168,   4, 1),
          apSubnetIP(255, 255, 255, 0);

#include <sWiFiFull.h>  // always under the optional #define xyz declarations 

void setup() {
  Serial.begin(115200);
  while (!Serial); delay(1000);
  Serial.println("\n");
  setupWiFi();  // is necessary on this place - init all your defined choices
  // your own code //
}

void loop() {
  loopWiFi();  // is necessary on this place - connection 'stays alive' control
  // your own code //
  yield();
}
