#ifndef _BMP280_H_
#define _BMP280_H_

#include <Wire.h>

uint8_t  s280addr = 0x76;  // or 0x77 I2C address
uint16_t dig_T1, dig_P1;   // reg values
int16_t  dig_T2, dig_T3, dig_P2, dig_P3, dig_P4, dig_P5, dig_P6, dig_P7, dig_P8, dig_P9;  // reg values T2->T3, P2->P9

bool s280begin(uint8_t addr = s280addr);
uint8_t s280read(uint8_t reg, uint8_t len, uint8_t *buf = NULL);
uint16_t s280compPar(bool first);
void s280getTPA(double *tpa, double seaLev = 1013.25);  // get values temp, press, alti

bool s280begin(uint8_t addr) {
  s280addr = addr;
  Wire.begin();
  if (s280read(0xD0, 1, 0) != 0x58) return false;
  dig_T1 =          s280compPar(true);  // first time read, then read the 24 TPcomp bytes
  dig_T2 = (int16_t)s280compPar(false);
  dig_T3 = (int16_t)s280compPar(false);
  dig_P1 =          s280compPar(false);
  dig_P2 = (int16_t)s280compPar(false);
  dig_P3 = (int16_t)s280compPar(false);
  dig_P4 = (int16_t)s280compPar(false);
  dig_P5 = (int16_t)s280compPar(false);
  dig_P6 = (int16_t)s280compPar(false);
  dig_P7 = (int16_t)s280compPar(false);
  dig_P8 = (int16_t)s280compPar(false);
  dig_P9 = (int16_t)s280compPar(false);
  Wire.beginTransmission(s280addr);
  Wire.write(0xF4); Wire.write(0x3F);
  Wire.endTransmission();
  return true;
}

uint8_t s280read(uint8_t reg, uint8_t len, uint8_t *buf) {
  Wire.beginTransmission(s280addr);
  Wire.write(reg);
  Wire.endTransmission();
  Wire.requestFrom(s280addr, len);
  if (len > 1) {
    for (uint8_t i = 0; i < len; i++) buf[i] = Wire.read();
    return 0;
  }
  else return Wire.read();
}

uint16_t s280compPar(bool first) {
  static uint8_t buf[24], p;
  if (first) {
    s280read(0x88, 24, buf); // I2C T-P-comp read buffer
    p = 0;
  }
  return buf[p++] + buf[p++] * 256;
}

void s280getTPA(double *tpa, double seaLev) {  // get values temp, press, alti
  int32_t adc_T, adc_P, t_fine;
  int64_t var1, var2, p = 0;
  static uint8_t buf[6];

  s280read(0xF7, 6, buf); // I2C adc read buffer
  adc_P = ((buf[p++] << 16) + (buf[p++] << 8) + buf[p++]) >> 4;
  adc_T = ((buf[p++] << 16) + (buf[p++] << 8) + buf[p++]) >> 4;
  // calc Temp
  var1 = ((((adc_T >> 3) - ((int32_t)dig_T1 << 1))) * ((int32_t)dig_T2)) >> 11;
  var2 = (((((adc_T >> 4) - ((int32_t)dig_T1)) * ((adc_T >> 4) - ((int32_t)dig_T1))) >> 12) * ((int32_t)dig_T3)) >> 14;
  t_fine = var1 + var2;
  tpa[0] = (double)((t_fine * 5 + 128) >> 8) / 100;

  // calc Press
  var1 = ((int64_t)t_fine) - 128000;
  var2 = var1 * var1 * (int64_t)dig_P6;
  var2 = var2 + ((var1 * (int64_t)dig_P5) << 17);
  var2 = var2 + (((int64_t)dig_P4) << 35);
  var1 = ((var1 * var1 * (int64_t)dig_P3) >> 8) + ((var1 * (int64_t)dig_P2) << 12);
  var1 = (((((int64_t)1) << 47) + var1)) * ((int64_t)dig_P1) >> 33;
  if (var1 == 0) tpa[1] = 0;
  else {
    p = 1048576 - adc_P;
    p = (((p << 31) - var2) * 3125) / var1;
    var1 = (((int64_t)dig_P9) * (p >> 13) * (p >> 13)) >> 25;
    var2 = (((int64_t)dig_P8) * p) >> 19;
    p = ((p + var1 + var2) >> 8) + (((int64_t)dig_P7) << 4);
    tpa[1] = (double)(p >> 8);
  }

  // calc Alti
  tpa[2] = 44330.0 * (1.0 - pow((tpa[1] / 100.0) / seaLev, 0.1903));
}

#endif
