////////////////// sDS18B20.h ///////////////////
//        Ewald sr, www.kendziorra.nl          //
//   a very simple DS18B20 mini "1-Wire" lib.  //
//  Nov 2020, completely free to edit and use  //
//---------------------------------------------//
//    use it as class for a single DS18B20     //
//              -----      -------             //
//        each sensor it's own IO-port         //
//         normal mode (NOT parasite)          //
// No other 1-wire or Dallas library is needed //
//---------------------------------------------//
//   tested and OK with an ESP32 DevkitC V4    //
////////////////// sDS18B20.h ///////////////////

#ifndef _S_DS18B20_H_
  #define _S_DS18B20_H_

  class sDS18B20 {
    private:
      uint8_t tPin;
      float tComp;
    public:
      float getTemp(uint8_t p, float c = 0.0);
      float getTemp();        // DS18B20 get temp in *C/F/K if error: return >=999.9 C(F/K);
    private:
      bool reset();           // return false if no presence detect was found, return true otherwise
      void wByte(uint8_t b);  // write a data uint8_t
      uint8_t rByte();        // read a data uint8_t
  };
  
  float sDS18B20::getTemp(uint8_t p, float c) {
    tPin = p;
    tComp = c;
    return getTemp();
  }
  
  float sDS18B20::getTemp() {
    if (!reset()) return 999.9;  // error
    wByte(0xCC); delay(  5);
    wByte(0x44); delay(750);
    reset();
    wByte(0xCC); delay(  5);
    wByte(0xBE);
    uint8_t b[2] = {rByte(), rByte()};
    int16_t t = b[1] << 0x8 | b[0];
    return ((float)t / 16.0) + tComp;  // temperature in Celcius
  }
  
  bool sDS18B20::reset(void) {
    uint8_t b1;
    pinMode(tPin, OUTPUT);
    noInterrupts();
    digitalWrite(tPin, LOW);
    delayMicroseconds(480);
    digitalWrite(tPin, HIGH);
    pinMode(tPin, INPUT);
    delayMicroseconds(70);
    b1 = (digitalRead(tPin) == 0);
    delayMicroseconds(410);
    interrupts();
    return b1;
  }
  
  void sDS18B20::wByte(uint8_t b0) {
    uint8_t b1;
    for (uint8_t loop = 0; loop < 8; loop++) {
      b1 = b0 & 0x01;
      pinMode(tPin, OUTPUT);
      noInterrupts();
      digitalWrite(tPin, LOW);
      if (b1) {
        delayMicroseconds(6);
        pinMode(tPin, INPUT);
        delayMicroseconds(64);
      }
      else {
        delayMicroseconds(60);
        pinMode(tPin, INPUT);
        delayMicroseconds(10);
      }
      interrupts();
      b0 >>= 1;
    }
  }

  uint8_t sDS18B20::rByte() {
    uint8_t b0 = 0, b1;
    for (uint8_t loop = 0; loop < 8; loop++) {
      b0 >>= 1;
      pinMode(tPin, OUTPUT);
      noInterrupts();
      digitalWrite(tPin, LOW);
      delayMicroseconds(6);
      pinMode(tPin, INPUT);
      delayMicroseconds(9);
      b1 = digitalRead(tPin) & 0x01;
      delayMicroseconds(55);
      interrupts();
      if (b1) b0 |= 0x80;
    }
    return b0;
  }
  
#endif
