#ifndef _SDST_
#define _SDST_

time_t dstStart,  // is set after calcDst(), calcDst(time_t)
       dstEnd;    //  ...
const uint16_t oneHour = 3600;
const uint32_t oneDay  = 86400,
               oneWeek = 604800;

void calcDst();                     // sDST calc, using time now()   (UTC)
void calcDst(time_t);               // sDST calc, using a other time (UTC)
bool isItDst(time_t, bool);   // return true/false - if true: use calcDst([time]) first
time_t localTime(time_t, bool);  // return local time - if true: use calcDst([time]) first
/** extra date/time functions **/
time_t calcTimeStamp(uint8_t, uint8_t, uint8_t, uint8_t, uint8_t, uint16_t);
uint16_t yearDay(time_t);
uint16_t yearDay(uint8_t, uint8_t, uint16_t);
bool leapYear(uint16_t);

void calcDst() {
  calcDst(now());
}

void calcDst(time_t t) {
  uint16_t y = year(t);
  /** fixed date? **/
  if (setDST.startMonthDay) {
    /** Month Day > 0: it's a fixed sDST **/
    dstStart = calcTimeStamp(setDST.startHour, setDST.startMinute, 0, setDST.startMonthDay, setDST.startMonth, y);
    dstEnd = calcTimeStamp(setDST.endHour, setDST.endMinute, 0, setDST.endMonthDay, setDST.endMonth, y);
  }
  /** flex date sDST **/
  /** last week 4/5 of the month? **/
  else {
    /** start **/
    if (setDST.startWeek == 5) {  // last week
      dstStart = calcTimeStamp(setDST.startHour, setDST.startMinute, 0, 1, setDST.startMonth + 1, y);  // get the first day of the next mondth
      dstStart -= oneDay;  // set last day of the month
      while (setDST.startWeekDay != weekday(dstStart)) dstStart -= oneDay;
    } 
    else {  // week 1-4
      dstStart = calcTimeStamp(setDST.startHour, setDST.startMinute, 0, 1, setDST.startMonth, y);  // get the first day of the next mondth
      dstStart += (setDST.startWeek - 1) * oneWeek;  // set the week of the month
      while (setDST.startWeekDay != weekday(dstStart)) dstStart += oneDay;
    }
    if (setDST.endWeek == 5) {
      /** end **/   
      dstEnd = calcTimeStamp(setDST.endHour, setDST.endMinute, 0, 1, setDST.endMonth + 1, y);  // get the first day of the next mondth
      dstEnd -= oneDay;  // set last day of the month
      while (setDST.endWeekDay != weekday(dstEnd)) dstEnd -= oneDay;
    }
    else {
      /** end **/
      dstEnd = calcTimeStamp(setDST.endHour, setDST.endMinute, 0, 1, setDST.endMonth, y);  // get the first day of the next mondth
      dstEnd += (setDST.endWeek - 1) * oneWeek;  // set the week of the month
      while (setDST.endWeekDay != weekday(dstEnd)) dstEnd += oneDay;
    }
  }
}

bool isItDst(time_t t, bool calc) {
  bool d = false;
  if (calc) calcDst(t);
  /** location north or south of the equator? **/
  if (setDST.hemisphere) {
    if (t >= dstStart && t < dstEnd) d = true;  // north
  }
  else if (!(t < dstStart && t >= dstEnd)) d = true;  // south
  return d;
}

time_t localTime(time_t t, bool calc) {
  return t + (uint32_t)(setDST.timezone * oneHour + (isItDst(t, calc) ? (setDST.dstoffset * oneHour) : 0)); 
}

time_t calcTimeStamp(uint8_t h, uint8_t m, uint8_t s, uint8_t D, uint8_t M, uint16_t Y) {  // get UNIX time_t Time Stamp from hh:mm:ss DD-MM-YY
  // UNIX timestamp = number of seconds since 1 Jan 00:00:00 1970
  if (Y < 100) Y += 2000;
  time_t t = 0;
  for (uint16_t i = 1970; i < Y; i++) t += yearDay(31, 12, i) * oneDay;
  return t + (yearDay(D, M, Y) - 1) * oneDay + (uint32_t)h * oneHour + (uint32_t)m * 60 + s;
}

uint16_t yearDay(time_t t) {  // year days from timestamp
  return yearDay(day(t), month(t), year(t));
}

uint16_t yearDay(uint8_t d, uint8_t m, uint16_t y) {  // year days from date numbers
  int monthdays[13] = {0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334};
  return monthdays[m - 1] + (monthdays[m - 1] > 31 && leapYear(y) ? 1 : 0) + d;
}

bool leapYear(uint16_t y) {  // leap year?
  uint16_t Y = y < 100 ? y + 2000 : y;
  return ((Y % 4 == 0 && Y % 100 != 0) || Y % 400 == 0);
}

#endif
