/** Ewald sr, aug 2020 www.kendziorra.nl 
 **  we use only ADC1-ports **
 *          ESP32 GPIO  
 * ADC1_CHANNEL_0: 36  
 * ADC1_CHANNEL_1: 37  
 * ADC1_CHANNEL_2: 38  
 * ADC1_CHANNEL_3: 39  
 * ADC1_CHANNEL_4: 32  
 * ADC1_CHANNEL_5: 33  
 * ADC1_CHANNEL_6: 34  
 * ADC1_CHANNEL_7: 35  
 * -  
 * ADC_WIDTH_BIT_9,  0-511  
 * ADC_WIDTH_BIT_10, 0-1023  
 * ADC_WIDTH_BIT_11, 0-2047  
 * ADC_WIDTH_BIT_12, 0-4095  
 * -  
 * ADC_ATTEN_DB_0,   0-1.1V  
 * ADC_ATTEN_DB_2_5, 0-1.5V  
 * ADC_ATTEN_DB_6,   0-2.2V  
 * ADC_ATTEN_DB_11,  0-3.9V  
 * -  
 * number of cycles per sample, default is 8, range is 1 - 255  
 * number of samples in the range, default is 1, range is 1 - 255
**/

#ifndef _S_ESP_ADC_H_
#define _S_ESP_ADC_H_

  #include <esp_adc_cal.h>
  
  int16_t sDEFAULT_VREF = 1100;  // default 1100
  adc_atten_t sADC_ATTEN_DB = ADC_ATTEN_DB_11;  // 0-3.9V
  adc_bits_width_t sADC_WIDTH_BIT = ADC_WIDTH_BIT_12; // 0-4095
  bool sIsInit = false;  // used for auto init
  
  esp_adc_cal_characteristics_t *adc_chars = new esp_adc_cal_characteristics_t;

  String sInitAdc(int16_t _DEFAULT_VREF = sDEFAULT_VREF,
                  adc_atten_t _ADC_ATTEN_DB = sADC_ATTEN_DB, adc_bits_width_t _ADC_WIDTH_BIT = sADC_WIDTH_BIT);
  uint32_t sAdcRead(uint8_t pin, uint8_t rep = 2);
  uint32_t sAdcVolt(uint8_t pin, uint8_t rep = 2);
  
  String sInitAdc(int16_t _DEFAULT_VREF, adc_atten_t _ADC_ATTEN_DB, adc_bits_width_t _ADC_WIDTH_BIT) {
    String str = "Default";
    sIsInit = true;
    sDEFAULT_VREF = _DEFAULT_VREF;
    sADC_ATTEN_DB = _ADC_ATTEN_DB;
    sADC_WIDTH_BIT = _ADC_WIDTH_BIT;
    adc1_config_width(sADC_WIDTH_BIT);
    esp_adc_cal_value_t val_type = esp_adc_cal_characterize(ADC_UNIT_1, sADC_ATTEN_DB, sADC_WIDTH_BIT, sDEFAULT_VREF, adc_chars);
    if (val_type == ESP_ADC_CAL_VAL_EFUSE_VREF) str = "eFuse Vref";
    else if (val_type == ESP_ADC_CAL_VAL_EFUSE_TP) str = "Two Point";
    return str;
  }
  
  uint32_t sAdcRead(uint8_t pin, uint8_t rep) {  // v==true: get mV else get ADC value
    uint32_t r = 0;
    if (!sIsInit) sInitAdc();  // use auto init with default values?
    adc1_channel_t cNr;
    switch (pin) {
      case (36):
        cNr = ADC1_CHANNEL_0;
        break;
      case (37):
        cNr = ADC1_CHANNEL_1;
        break;
      case (38):
        cNr = ADC1_CHANNEL_2;
        break;
      case (39):
        cNr = ADC1_CHANNEL_3;
        break;
      case (32):
        cNr = ADC1_CHANNEL_4;
        break;
      case (33):
        cNr = ADC1_CHANNEL_5;
        break;
      case (34):
        cNr = ADC1_CHANNEL_6;
        break;
      case (35):
        cNr = ADC1_CHANNEL_7;
        break;
    }
    adc1_config_channel_atten(cNr, sADC_ATTEN_DB);
    for (uint8_t i = 0; i < rep; i++) {
      r += adc1_get_raw(cNr);
      delay(20);
    }
    return r / rep;
  }

  uint32_t sAdcVolt(uint8_t pin, uint8_t rep) {
    return esp_adc_cal_raw_to_voltage(sAdcRead(pin, rep), adc_chars);
  }

#endif
