/**  sMideaIR.h: A very small and simple to use Infrared Emitter AC control
  *  Totally free to use and modify. Ewald  Kendziorra sr, 2020, www.kendziorra.nl 
  *  -----------------------------------------------------------------------------  
  *  values: from my Midea Comfee remote control
  *  decoded with my "receiverIR.ino"  (on my web site)
  *  read: DECODING INFRARED PROTOCOLS (on my web site)
  *  read: MIDEA AC IR PROTOCOL        (on my web site)
  **/// use a OPEN-SMART 38KHz Infrared Emitter //
     //   NO NEED FOR A INFRARED LIBRARY ;=)    //
     //=========================================//
#ifndef _S_MIDEA_IR_H
  #define _S_MIDEA_IR_H
  
  byte _irPin = 4;         // use a digital 'OUT' pin
  const uint16_t  T1L = 540, T8L = 4400,                             // LOW times
                  T1P = 540, T3P = 1600, T8P = 4400, T10P = 5200;    // PULSED times
  // position, command and value labels //
  enum {byte1 = 0, byte2, byte3 = 4, byte4, byte5 =  8, byte6,        // cmd byte positions in        cmdBytes[]
        invb1 = 2, invb2, invb3 = 6, invb4, invb5 = 10, invb6,        //   "        "
       lowFan = 0, midFan, highFan, autoFan, offFan,                  // fan mode positions in        fanVal[]
        tmp17 = 0, tmp18, tmp19, tmp20, tmp21, tmp22, tmp23, tmp24,   // temp positions in            tempVal[]
                   tmp25, tmp26, tmp27, tmp28, tmp29, tmp30, tmpOff,  //   "         "
      coolMod = 0, heatMod, autoMod, fanMod, ventMod, dehumMod,       // mode positions in            modVal[]
         onAC = 0, offAC, deflectAC, oscilAC, turboAC, sleepAC,       // spesial command positions in cmdBytes[]
                   cleanAC, ledAC, ecoOnAC, ecoOffAC, ionizeAC        //   "         "
  };
  const byte nbytes = 12,  // number of bytes (6 + 6 inverted)
          fanVal[5] =  {0x9, 0x5, 0x3, 0xB, 0xE},                 // low, mid, high, auto, off
        tempVal[15] =  {0x0, 0x1, 0x3, 0x2, 0x6, 0x7, 0x5, 0x4,   // 17, 18, 19->30*C and off
                             0xC, 0xD, 0x9, 0x8, 0xA, 0xB, 0xE},  //    "        "
          modeVal[6] =  {0x0, 0xC, 0x8, 0x4, 0xF, 0xD},           // cool, heat, auto, fan, ventilate, dehumidify
     spcBytes[11][6] = {{0xB, 0x2, 0xB, 0xF, 0x4, 0x0},  // on         onAC (standard command values on init)
                        {0xB, 0x2, 0x7, 0xB, 0xE, 0x0},  // off        offAC
                        {0xB, 0x2, 0x0, 0xF, 0xE, 0x0},  // deflector  deflectAC
                        {0xB, 0x5, 0xF, 0x5, 0xB, 0x2},  // oscillate  oscilAC
                        {0xB, 0x5, 0xF, 0x5, 0xA, 0x2},  // turbo      turboAC
                        {0xB, 0x2, 0xE, 0x0, 0x0, 0x3},  // sleep      sleepAC
                        {0xB, 0x5, 0xF, 0x5, 0xA, 0xA},  // clean      cleanAC
                        {0xB, 0x5, 0xF, 0x5, 0xA, 0x5},  // led        ledAC
                        {0xB, 0x5, 0xF, 0x5, 0x8, 0x2},  // eco on     ecoOnAC
                        {0xB, 0x5, 0xF, 0x5, 0x8, 0x3},  // eco off    ecoOffAC
                        {0xB, 0x5, 0xF, 0x5, 0xA, 0x9}}; // ionize     ionizeAC
  byte cmdBytes[nbytes] = {0},  // command bytes buffer
     saveStdCmd[nbytes] = {0};  // last standard command bytes buffer
  boolean lastStd = false;      // true after first 'stdCmd(n, n, n)' command
  
  void mideaBegin(byte _pin);
  void stdCmd(byte _fan, byte _tmp, byte _mode);  // standardCommand, use the 'enum' values
  void spcCmd(byte _cmd);                         // SpesialCommand, use the 'enum' values
  void _spcCmd(byte _cmd);  // private
  void _setInvert();  // private  -  Midea expected also INVERTED 16bits values
  void _sendCmd();    // private  -  Midea expected 2x the same command
  void _sendIt(uint16_t t1, uint16_t t2);  // private  -  NO NEED FOR A INFRARED LIBRARY
  
  void mideaBegin(byte _pin) {
    _irPin = _pin;
    _spcCmd(onAC);
    _setInvert();
    memcpy(saveStdCmd, cmdBytes, nbytes);
    pinMode(_irPin, OUTPUT);
    digitalWrite(_irPin, LOW);
  }
  
  void stdCmd(byte _fan, byte _tmp, byte _mode) {
    cmdBytes[byte1] = 0xB;
    cmdBytes[byte2] = 0x2;
    cmdBytes[byte3] = fanVal[_fan];
    cmdBytes[byte4] = 0xF;
    cmdBytes[byte5] = tempVal[_tmp];
    cmdBytes[byte6] = modeVal[_mode];
    _setInvert();
    memcpy(saveStdCmd, cmdBytes, nbytes);
    lastStd = true;
    _sendCmd();  
  }
  
  void spcCmd(byte _cmd) {
    if ((_cmd == onAC) && lastStd) memcpy(cmdBytes, saveStdCmd, nbytes);
    else {
      _spcCmd( _cmd);
      _setInvert();  
    }
    _sendCmd();  
  }

  void _spcCmd(byte _cmd) {
    cmdBytes[byte1] = spcBytes[_cmd][0];
    cmdBytes[byte2] = spcBytes[_cmd][1];
    cmdBytes[byte3] = spcBytes[_cmd][2];
    cmdBytes[byte4] = spcBytes[_cmd][3];
    cmdBytes[byte5] = spcBytes[_cmd][4];
    cmdBytes[byte6] = spcBytes[_cmd][5];
  }
  
  void _setInvert() {  /** private **/
    cmdBytes[invb1] = ~cmdBytes[byte1];
    cmdBytes[invb2] = ~cmdBytes[byte2];
    cmdBytes[invb3] = ~cmdBytes[byte3];
    cmdBytes[invb4] = ~cmdBytes[byte4];
    cmdBytes[invb5] = ~cmdBytes[byte5];
    cmdBytes[invb6] = ~cmdBytes[byte6];
  }
  
  void _sendCmd() {  /** private **/
    for (byte i = 0; i < 2; i++) {  // Midea send 2x the same command
      _sendIt(T8L, T8P);  // START stream 
      for (byte j = 0; j < nbytes; j++) {  // extract the individual bits and send '1' or '0' pulsed time
        for (byte k = 0; k < 4; k++) _sendIt(T1L, ((cmdBytes[j] << k) & 0x8  ? T3P : T1P)); 
      }
      _sendIt(T1L, T10P);  // END stream
    }
    _sendIt(T1L, T10P);  // extra END stream 
  }
  
  void _sendIt(uint16_t t1, uint16_t t2) {  /** private **/ // 38KHz PULSED by OPEN-SMART Infrared Emitter
    digitalWrite(_irPin, HIGH);
    delayMicroseconds(t1);
    digitalWrite(_irPin, LOW);
    delayMicroseconds(t2);
  }

#endif
