#ifndef _S_THERMISTOR_H_
#define _S_THERMISTOR_H_

  class sThermistors {
    public:
      float getTemp();
  };
    
  float sThermistors::getTemp() {
    /** ESP32 ADC chanels/analogread are not stable so we use buffers to smooth out the result **/ 
    static bool isInit = false;
    static double A, B, C;
    const uint8_t bufLen = 20;  // minimal: 16
    uint64_t adcBuf[bufLen], adc = 0;
    if (!isInit) {
      // calc Steinhart-Hart ABC
      double
        L1 = log((double)ntcR[0] * calibr),
        L2 = log((double)ntcR[1] * calibr),
        L3 = log((double)ntcR[2] * calibr),
        Y1 = 1.0 / ((double)ntcT[0] + 273.15),
        Y2 = 1.0 / ((double)ntcT[1] + 273.15),
        Y3 = 1.0 / ((double)ntcT[2] + 273.15);
      Y2 = ((Y2 - Y1)/(L2 - L1));
      Y3 = ((Y3 - Y1)/(L3 - L1));
      C = ((Y3 - Y2) / (L3 - L2)) * pow((L1 + L2 + L3), -1.0);
      B = Y2 - C * (pow(L1, 2.0) + L1 * L2 + pow(L2, 2.0));
      //A = Y1 - B + pow(L1, 2.0) * C) * L1;
      A = Y1 - L1 * (B + C * pow(L1, 2.0));
      isInit = true;
    }
    for (uint8_t i = 0; i < bufLen; i++) {
      adcBuf[i] = analogRead(tempPin);
      yield();
    }
    for (uint8_t i = 0; i < bufLen - 1; i++) {  // a simple bubble sort
      for (uint8_t j = 0; j < bufLen - i - 1; j++) {
        if (adcBuf[j] > adcBuf[j + 1]) {
          uint64_t v = adcBuf[j];
          adcBuf[j] = adcBuf[j + 1];
          adcBuf[j + 1] = v;
        }
      }
    }
    for (uint8_t i = 3; i < bufLen - 3; ++i) adc += adcBuf[i];  // strip 3-lowest and 3-highest value
    adc /= (bufLen - 6);
    double R = ntcS * (4095.0 / (double)adc - 1.0),
           L = log(R),
           T = 1.0 / (A + B * L + C * pow(L, 3.0));
    #ifdef PRINT_RESISTANCE
      Serial.printf("Resistance: %u", (uint32_t)R);  //##### get the resistance at a given temperature #####
    #endif  
    return (float)T - 273.15;
  }
#endif
