#ifndef SIMPLETIMERS_H
#define SIMPLETIMERS_H

#include <Arduino.h>

class sTimers {
   public:
    /** add a timer **/
    sTimers(uint32_t ms, bool start);
    /** start/stop a timer **/
    void tmrStart(bool start);
    /** stop timer, set time, (start timer) **/
    void tmrReset(uint32_t ms, bool start);
    /** get interval in millis **/
    uint32_t tmrMs();
    /** timers time? **/
    bool tmrTime();

  private:
    uint32_t tmrOffset = 0,
             tmrMillis = 0;
    bool tmrRun = false;    
    /** get the number of millis after _start with overflow correction **/
    uint32_t getMillis(uint32_t _start);
};

/** add a timer */
sTimers::sTimers(uint32_t ms, bool start) {
  tmrOffset = ms;
  tmrStart(start);
}

/** start/stop a timer **/
void sTimers::tmrStart(bool start) {
  if (start) {
    tmrRun = true;
    tmrMillis = millis();  
  }
  else tmrRun = false;
}

/** stop timer, set time, (start timer) **/
void sTimers::tmrReset(uint32_t ms, bool start) {
  tmrStart(false);
  tmrOffset = ms;
  tmrStart(start);
}

/** get interval in millis **/
uint32_t sTimers::tmrMs() {
  return tmrOffset;
}

/** timers in millis **/
bool sTimers::tmrTime() {
  bool f = false;
  if (tmrRun && (getMillis(tmrMillis) >= tmrOffset)) {
    f = true;
    tmrMillis = millis();
  }
  return f;
}

/** get the millis after _start with overflow correction **/
uint32_t sTimers::getMillis(uint32_t _start) {  
  uint32_t _stop = millis();
  if (_stop < _start) return (uint32_t)4294967295 - _start + _stop + 1;
  else return _stop - _start;
}

#endif
