﻿/**  sWiFiFull: a very easy to use ESP32 WiFi library.
  *  - Automatic initialization and connection
  *  - 'stays alive' control
  *  - selectable functional options
  *      STATION - OTA - WEBSOCKET -
  *                DATE/TIME/SUN - WEATHER
  *      ACCESS POINT - ESP-NOW
  *  Totally free to use and modify
  *  Ewald Kendziorra sr, 2021, www.kendziorra.nl 
  *****************************************************/ 
#ifndef _S_WIFIFULL_H_
#define _S_WIFIFULL_H_
 
  #include <esp_wifi.h>
  #include <WiFi.h>
  #ifdef USE_NOW
    #include <esp_now.h>
  #endif
  #ifdef USE_WS
    #include <WebSocketsServer.h>  // used library: https://github.com/Links2004/arduinoWebSockets
  #endif
  #ifdef USE_OTA
    #include <ArduinoOTA.h>
  #endif
  #ifdef USE_TIME
    #include "sDateTimeSun.h"         // (simpleLib)
  #endif
  #ifdef USE_WTHR
    #include "sOpenweathermap.h"   // (simpleLib)
  #endif

  bool isInit = false;
  
  #ifdef USE_NOW
    uint8_t nowNodeMAC[] = {nowAdrMAC, 0, 2, 3, 4, 5};
    esp_now_peer_info_t slave;
    const esp_now_peer_info_t *peer = &slave;
    bool nowSendData(uint8_t _mac, const char *_msg);
    void nowOnDataRecv(const uint8_t *mac, const uint8_t *data, int len);
    bool nowVal = false;
    String nowBuf = "";  // we use a String so do not suffer from a character array length problem
  #endif
  #if defined(USE_NOW) || defined(USE_WS)
    uint8_t parseFldVal(char * mesage);
  #endif
  #ifdef USE_WS
    WebSocketsServer webSocket = WebSocketsServer(wsPort, wsURI, wsProtocol);
    void webSocketEvent(uint8_t num, WStype_t type, uint8_t *payload, size_t len);
    bool wsVal = false;
    uint8_t wsClients = 0;  // the number of clients
    String  wsBuf = "";  // we use a String so do not suffer from a character array length problem
  #endif

  void setupWiFi();
  void loopWiFi();
  #ifdef USE_STA
    void testWiFi();
  #endif

  void setupWiFi() {  // return 0:NOT connected, 1:connected, 2:already connected
    WiFi.disconnect();
    WiFi.mode(WIFI_OFF);
    WiFi.setSleep(false);
    #if defined(USE_STA) && defined(USE_AP)
      WiFi.mode(WIFI_AP_STA);
    #else
      #ifdef USE_STA
        WiFi.mode(WIFI_STA);
      #else
        WiFi.mode(WIFI_AP);
      #endif
    #endif    
    #ifdef USE_AP
      #ifdef USE_APIP
        WiFi.softAPConfig(apIP, apGatewayIP, apSubnetIP);
        delay(200);
      #endif    
      #ifdef USE_NOW
        char c[3] = {nowAddress[0], nowAddress[1]};
        nowNodeMAC[1] = atoi(c);  // building #
//        esp_wifi_set_mac(ESP_IF_WIFI_AP, nowNodeMAC);  // before version < 1.0.5
        esp_wifi_set_mac(WIFI_IF_AP, nowNodeMAC);  // after version > 1.0.5
     #endif
      int8_t apOpt[] = {1, 0, 4};  // channel, hiden (false), max connections
      #ifdef USE_APCHAN
        apOpt[0] = apChannel;
      #endif
      #ifdef USE_APHID
        apOpt[1] = apHiden;
      #endif
      #ifdef USE_APMAXCON
        apOpt[2] = apMaxCon;
      #endif  
      WiFi.softAP(apSsid, apPassword, apOpt[0], apOpt[1], apOpt[2]);
      delay(500);
      wifi_config_t conf;
      esp_wifi_get_config(WIFI_IF_AP, &conf);
      Serial.println("AP           mac: " + String(WiFi.softAPmacAddress()));
        Serial.print("              IP: "); Serial.println(WiFi.softAPIP());
        Serial.print("         Channel: "); Serial.println(conf.ap.channel);
        Serial.print("           hiden: "); Serial.println(conf.ap.ssid_hidden ? 'Y' : 'N');
        Serial.print("            SSID: "); Serial.println((char *)conf.ap.ssid);
        Serial.print("              PW: "); Serial.println((char *)conf.ap.password);
        Serial.print("  max connection: "); Serial.println(conf.ap.max_connection);
    #endif      
    // STA after AP mode!!! otherwise station will not work as desired
    #ifdef USE_STA 
      #ifdef USE_STAIP
        #ifdef USE_STADNS
          WiFi.config(staIP, staGatewayIP, staSubnetIP, staPrimDNS, staSecDNS);
        #else
          WiFi.config(staIP, staGatewayIP, staSubnetIP);
        #endif   
        delay(200);        
      #endif     
      #ifdef USE_STACHAN
        WiFi.begin(staSsid, staPassword, staChannel);
      #else
        WiFi.begin(staSsid, staPassword);
      #endif
      Serial.print("WiFi connect .");
      time_t t2 = millis() + 30000;
      while (1) {
        if (WiFi.isConnected()) { Serial.print("Yes!");break;}
        if (millis() > t2) {
          Serial.print("NO!!!\nNow we use 'loopWiFi();' to connect!\n (put it in 'void loop();')");
          break;
        }
        Serial.print(".");
        delay(500);
      }
      Serial.println();
      #ifdef USE_LR
        esp_wifi_set_protocol(WIFI_IF_AP, WIFI_PROTOCOL_LR);
      #endif
      delay(500);
      Serial.println("STA    mac: " + String(WiFi.macAddress()));
        Serial.print("       IP : "); Serial.println(WiFi.localIP());
        Serial.print("   Channel: "); Serial.println(WiFi.channel());
      #ifdef USE_WS
        Serial.printf("      Port: %u\n  Protocol: %s\n", wsPort, wsProtocol);
      #endif
      #ifdef USE_AP  // AP_STA must have the same channel number
        if (WiFi.channel() != conf.ap.channel) {
          conf.ap.channel = WiFi.channel();
          esp_wifi_set_config(WIFI_IF_AP, &conf);
          esp_wifi_get_config(WIFI_IF_AP, &conf);                            
          Serial.print("AP channel changed to: "); Serial.println(conf.ap.channel);
        }
      #endif
    #endif
    #ifdef USE_NOW
      if(esp_now_init() == ESP_OK) {
        Serial.print("Added Peer: ");
        for (byte i = 0; i < nowMaxSlaves; i++) {
          nowNodeMAC[1] = i;
          memcpy(&slave.peer_addr, &nowNodeMAC, 6);
          slave.channel = apChannel;
          slave.encrypt = 0;
          if( esp_now_add_peer(peer) == ESP_OK) Serial.printf("%u%s", i, (i < (nowMaxSlaves - 1) ? ", " : "\n"));
        }
        esp_now_register_recv_cb(nowOnDataRecv);
      }
    #endif      
    #ifdef USE_STA
      #ifdef USE_OTA
        ArduinoOTA.setHostname(myOtaID);
        Serial.printf("OTAname: %s\n", myOtaID);
        ArduinoOTA.begin();
      #endif      
      #ifdef USE_WS
        webSocket.begin();
        webSocket.onEvent(webSocketEvent);  // not a break
      #endif  
      if (!WiFi.isConnected()) testWiFi();
      #ifdef USE_TIME
        Serial.print("NTP connect .");
        while (!sGetNTP(timeZone)) {
          Serial.print(".");
          delay(1000);  // no time: keep trying
        }
        Serial.println("Yes!");
      #endif
      #ifdef USE_WTHR
        weatherInit(lat, lon, units, key);  // init and fill the weather structure - default: printInfo = false
      #endif
    #endif
    isInit = true;
  }

  void loopWiFi() {
    if (!isInit) setupWiFi();
    #ifdef USE_NOW
      if (esp_now_init() != ESP_OK) setupWiFi();
    #endif
    #ifdef USE_STA
      if (!WiFi.isConnected()) testWiFi();
      if (WiFi.isConnected()) {
        #ifdef USE_OTA
          ArduinoOTA.handle();
        #endif      
        #ifdef USE_WS
          webSocket.loop(); 
        #endif      
        #ifdef USE_TIME
          if (syncTime > 0) {
            char *_tz = getenv("TZ");  // read used timzone
            sEpoch ep = sNow();
            if ((ep.sec % syncTime) == 0) sGetNTP(_tz);  // update time every intervalNTP sec.
          }
        #endif
      }
    #endif      
  }

  #ifdef USE_STA
    void testWiFi() {
      WiFi.begin();
      time_t t = millis() + 20000;
      while (millis() < t) {
        if (WiFi.isConnected()) {
          Serial.print("STA    IP : "); Serial.println(WiFi.localIP());
          Serial.print("   Channel: "); Serial.println(WiFi.channel());
          break;
        }
        delay(500);
      }
    }
  #endif
  
  #ifdef USE_NOW
    bool nowSendData(uint8_t _mac, const char *_msg) {
        nowNodeMAC[1] = _mac;
        if( esp_now_send(nowNodeMAC, (uint8_t*)_msg, strlen(_msg) + 1) == ESP_OK) return true;
        else return false;
    }

    void nowOnDataRecv(const uint8_t *mac, const uint8_t *payload, int len) {
      nowBuf = String((char*)payload);  // use this data
      nowVal = true;
    }
  #endif
  
  #ifdef USE_WS
    void webSocketEvent(uint8_t num, WStype_t type, uint8_t *payload, size_t len) {
      switch (type) {
        case WStype_DISCONNECTED: {
          if (wsClients) wsClients--;
          break;
        }
        case WStype_CONNECTED: {
          wsClients++;
          webSocket.sendTXT(num, "Connected");
          break;
        }
        case WStype_TEXT: {
          if (len) {
            wsBuf = String((char*)payload);  // use this data
            wsVal = true;
          }
          break;
        }
      }
    } 
  #endif

  #ifdef USE_PARSER
    /** parse the received message to fieldnames an fieldValues **/ 
    uint8_t parseFldVal(char * mesage) {
      uint8_t n = 0;
      char * v;
      /** get the number of fields **/
      v = strchr(mesage, '|');
      while (v != NULL) {
        n++;
        v = strchr(v + 1, '|');
      }
      if (n > 3 && (n % 2 != 0)) { // <= 3:no data, 'n' not neg. wrong format
        n = (n - 1) / 2;;
        uint8_t i = 0;
        /** get the fields **/
        v = strtok (mesage, "|");
        while (v != NULL) {
          strcpy(fldIn[i], v);
          v = strtok (NULL, "|");
          strcpy(valIn[i++], v);
          v = strtok (NULL, "|");
        }
      }
      else n = 0;
      return n;
    }
  #endif
#endif
