#ifndef _S_I2C_H_
#define _S_I2C_H_

#include <Wire.h>

bool sI2Cinit = false;
void sI2Cbegin();
int16_t sI2Cread(uint8_t _len, uint8_t* _buf);
void sI2Cwrite(uint8_t _len, uint8_t* _buf);
uint32_t sI2Cmillis(uint32_t _start);  

int16_t sI2Cread(uint8_t _len, uint8_t* _buf) {
  // _len:   the number of bytes to read
  // input:  device.address in _buf[0] and reg.address in _buf[l]
  // output: _buf[0...], return: _buf[0].value or -1 if timeout
  if (!sI2Cinit) sI2Cbegin();
  bool isOK = true;
  sI2Cwrite(1, _buf);  // set device.address and reg.address in the I2C write function
  Wire.requestFrom(_buf[0], _len);
  for (uint8_t i = 0; i <= _len; i++) _buf[i] = 0;  // clear _buffer
  uint32_t ms = millis();
  while (Wire.available() < _len) {
    if (sI2Cmillis(ms) > (_len * 100)) {isOK = false; break;}  // timeout: return -1
  }
  if (isOK) {
    uint8_t n = 0;
    while (Wire.available()) _buf[n++] = Wire.read();
    return _buf[0];  // return first byte and _buffer
  }
  else return -1;
}

void sI2Cwrite(uint8_t _len, uint8_t* _buf) {
  // device.address in _buf[0], reg.address in _buf[1] and data byte(s) to write _buf[2...]
  if (!sI2Cinit) sI2Cbegin();
  Wire.beginTransmission(_buf[0]);
  for (uint8_t i = 1; i <= _len; i++) Wire.write(_buf[i]);
  Wire.endTransmission(false);
}

void sI2Cbegin() {
  Wire.begin();
  Wire.setClock(400000);
  sI2Cinit = true;
}

/** get the millis after _start with overflow correction **/
uint32_t sI2Cmillis(uint32_t _start) {  
  uint32_t _stop = millis();
  if (_stop < _start) return (uint32_t)4294967295 - _start + _stop + 1;
  else return _stop - _start;
}

#endif
